<?php
/*
 * ibr_status_code_arrays.php
 * 
 * Copyright 2012 Kevin McCormick Longview, Texas
 * 
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 or later.  You should have 
 * received a copy of the GNU General Public License along with this program; 
 * if not, write to the Free Software Foundation, Inc., 
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *  <http://opensource.org/licenses/gpl-license.php>
 * 
 * 
 * @link http://www.wpc-edi.com
 * @author Kevin McCormick
 * @link: http://www.open-emr.org
 * @package OpenEMR
 * @subpackage ediHistory
 */

/**
 * retrieve descriptions of claim status codes
 * 
 * there are a multitude of codes and many will never be used in healthcare claim
 * status responses, but which ones are superfluous?
 * 
 * There are get_ methods only
 */
class status_code_arrays {
	
	private $STC_Category_Code;
	private $STC_HCSDR_Code;
	private $STC_Status_Code;
	private $STC_Entity_Code;
	private $STC_IDType_Code;
	
	/**
	 * ID Type codes
	 * 
	 * @param string code
	 * @return array 0=>code, [1]=>description, [2]=>empty, [3]=>empty  [4]=>empty
	 */
	public function get_STC_IDType_Code( $code ) {
		// 0 => code, [1] => description, [2] => dates, [3] => notes  (empty, none available)
		// contains "Unknown" for codes for which I did not find definitions
		if ( is_array( $this->STC_IDType_Code )) {
			if (array_key_exists($code, $this->STC_IDType_Code)){
				return $this->STC_IDType_Code[$code];
			} else {
				return array($code, "Not found in STC Category codes table", "", "", "");
			}
		} else {
			$this->STC_IDType_Code = array (
			'PI'=> array('PI', 'Payor Identification', '', '', ''),
			'XV'=> array('XV', 'Centers for Medicare and Medicaid Services PlanID', '', '', ''),
			'46'=> array('46', 'Electronic Transmitter Identification Number',  '', '', ''),
			'24'=> array('24', 'Employers Identification Number',  '', '', ''),
			'II'=> array('II', 'Standard Unique Health Identifier', '', '', ''), 
			'MI'=> array('MI', 'Member Identification Number', '', '', ''),
			'FI'=> array('FI', 'Federal Taxpayers Identification Number', '', '', ''),
			'SV'=> array('SV', 'Service Provider Number', '', '', ''),
			'XX'=> array('XX', 'Centers for Medicare and Medicaid Services NationalProvider Identifier', '', '', '')
			);
			if (array_key_exists($code, $this->STC_IDType_Code)){
				return $this->STC_IDType_Code[$code];
			} else {
				return array($code, "Not found in STC ID Type codes table", "", "", "");
			}
		}
	} // end get_STC_IDType_Code
			
	/**
	 * entity identifier code  --code source 237
	 * 
	 * Entity Codes from http://ushik.ahrq.gov/ and CMS 276/277 Side-by-Side
	 * 
	 * @param string code
	 * @return array 0=>code, [1]=>description, [2]=>empty, [3]=>empty  [4]=>empty
	 */	 
	public function get_STC_Entity_Code( $code ) {
		// 0 => code, [1] => description, [2] => '', [3] => '',  [4] => ''
		if ( is_array( $this->STC_Entity_Code )) {
			if (array_key_exists($code, $this->STC_Entity_Code)){
				return $this->STC_Entity_Code[$code];
			} else {
				return array($code, "Not found in STC Category codes table", "", "", "");
			}
		} else {
			$this->STC_Entity_Code = array(	
			'03' => array('03', 'Dependent', '', '', ''),
			'13' => array('13', 'Contracted Service Provider', '', '', ''),
			'17' => array('17', 'Consultant´s Office', '', '', ''),
			'1E' => array('1E', 'Health Maintenance Organization (HMO)', '', '', ''),
			'1G' => array('1G', 'Oncology Center', '', '', ''),
			'1H' => array('1H', 'Kidney Dialysis Unit', '', '', ''),
			'1I' => array('1I', 'Preferred Provider Organization (PPO)', '', '', ''),
			'1O' => array('1O', 'Acute Care Hospital', '', '', ''),
			'1P' => array('1P', 'Provider', '', '', ''),
			'1Q' => array('1Q', 'Military Facility', '', '', ''),
			'1R' => array('1R', 'University, College or School', '', '', ''),
			'1S' => array('1S', 'Outpatient Surgicenter', '', '', ''),
			'1T' => array('1T', 'Physician, Clinic or Group Practice', '', '', ''),
			'1U' => array('1U', 'Long Term Care Facility', '', '', ''),
			'1V' => array('1V', 'Extended Care Facility', '', '', ''),
			'1W' => array('1W', 'Psychiatric Health Facility', '', '', ''),
			'1X' => array('1X', 'Laboratory', '', '', ''),
			'1Y' => array('1Y', 'Retail Pharmacy', '', '', ''),
			'1Z' => array('1Z', 'Home Health Care', '', '', ''),
			'28' => array('28', 'Subcontractor', '', '', ''),
			'2A' => array('2A', 'Federal, State, County or City Facility', '', '', ''),
			'2B' => array('2B', 'Third-Party Administrator', '', '', ''),
			'2D' => array('2D', 'Miscellaneous Health Care Facility', '', '', ''),
			'2E' => array('2E', 'Non-Health Care Miscellaneous Facility', '', '', ''),
			'2I' => array('2I', 'Church Operated Facility', '', '', ''),
			'2K' => array('2K', 'Partnership', '', '', ''),
			'2P' => array('2P', 'Public Health Service Facility', '', '', ''),
			'2Q' => array('2Q', 'Veterans Administration Facility', '', '', ''),
			'2S' => array('2S', 'Public Health Service Indian Service Facility', '', '', ''),
			'2Z' => array('2Z', 'Hospital Unit of an Institution (prison hospital, college infirmary, etc.)', '', '', ''),
			'30' => array('30', 'Service Supplier', '', '', ''),
			'36' => array('36', 'Employer', '', '', ''),
			'3A' => array('3A', 'Hospital Unit Within an Institution for the Mentally Retarded', '', '', ''),
			'3C' => array('3C', 'Tuberculosis and Other Respiratory Diseases Facility', '', '', ''),
			'3D' => array('3D', 'Obstetrics and Gynecology Facility', '', '', ''),
			'3E' => array('3E', 'Eye, Ear, Nose and Throat Facility', '', '', ''),
			'3F' => array('3F', 'Rehabilitation Facility', '', '', ''),
			'3G' => array('3G', 'Orthopedic Facility', '', '', ''),
			'3H' => array('3H', 'Chronic Disease Facility', '', '', ''),
			'3I' => array('3I', 'Other Specialty Facility', '', '', ''),
			'3J' => array('3J', 'Children´s General Facility', '', '', ''),
			'3K' => array('3K', 'Children´s Hospital Unit of an Institution', '', '', ''),
			'3L' => array('3L', 'Children´s Psychiatric Facility', '', '', ''),
			'3M' => array('3M', 'Children´s Tuberculosis and Other Respiratory Diseases Facility', '', '', ''),
			'3N' => array('3N', 'Children´s Eye, Ear, Nose and Throat Facility', '', '', ''),
			'3O' => array('3O', 'Children´s Rehabilitiaion Facility', '', '', ''),
			'3P' => array('3P', 'Children´s Orthopedic Facility', '', '', ''),
			'3Q' => array('3Q', 'Children´s Chronic Disease Facility', '', '', ''),
			'3R' => array('3R', 'Children´s Other Specialty Facility', '', '', ''),
			'3S' => array('3S', 'Institution for Mental Retardation', '', '', ''),
			'3T' => array('3T', 'Alcoholism and Other Chemical Dependency Facility', '', '', ''),
			'3U' => array('3U', 'General Inpatient Care for AIDS/ARC Facility', '', '', ''),
			'3V' => array('3V', 'AIDS/ARC Unit', '', '', ''),
			'3W' => array('3W', 'Specialized Outpatient Program for AIDS/ARC', '', '', ''),
			'3X' => array('3X', 'Alcohol/Drug Abuse or Dependency Inpatient Unit', '', '', ''),
			'3Y' => array('3Y', 'Alcohol/Drug Abuse or Dependency Outpatient Services', '', '', ''),
			'3Z' => array('3Z', 'Arthritis Treatment Center', '', '', ''),
			'36' => array('36', 'Employer', '', '', ''),
			'40' => array('40', 'Receiver', '', '', ''),
			'43' => array('43', 'Claimant Authorized Representative', '', '', ''),
			'44' => array('44', 'Data Processing Service Bureau', '', '', ''),
			'4A' => array('4A', 'Birthing Room/LDRP Room', '', '', ''),
			'4B' => array('4B', 'Burn Care Unit', '', '', ''),
			'4C' => array('4C', 'Cardiac Catherization Laboratory', '', '', ''),
			'4D' => array('4D', 'Open-Heart Surgery Facility', '', '', ''),
			'4E' => array('4E', 'Cardiac Intensive Care Unit', '', '', ''),
			'4F' => array('4F', 'Angioplasty Facility', '', '', ''),
			'4G' => array('4G', 'Chronic Obstructive Pulmonary Disease Service Facility', '', '', ''),
			'4H' => array('4H', 'Emergency Department', '', '', ''),
			'4I' => array('4I', 'Trauma Center (Certified)', '', '', ''),
			'4J' => array('4J', 'Extracorporeal Shock-Wave Lithotripter (ESWL) Unit', '', '', ''),
			'4L' => array('4L', 'Genetic Counseling/Screening Services', '', '', ''),
			'4M' => array('4M', 'Adult Day Care Program Facility', '', '', ''),
			'4N' => array('4N', 'Alzheimer´s Diagnostic/Assessment Services', '', '', ''),
			'4O' => array('4O', 'Comprehensive Geriatric Assessment Facility', '', '', ''),
			'4P' => array('4P', 'Emergency Response (Geriatric) Unit', '', '', ''),
			'4Q' => array('4Q', 'Geriatric Acute Care Unit', '', '', ''),
			'4R' => array('4R', 'Geriatric Clinics', '', '', ''),
			'4S' => array('4S', 'Respite Care Facility', '', '', ''),
			'4U' => array('4U', 'Patient Education Unit', '', '', ''),
			'4V' => array('4V', 'Community Health Promotion Facility', '', '', ''),
			'4W' => array('4W', 'Worksite Health Promotion Facility', '', '', ''),
			'4X' => array('4X', 'Hemodialysis Facility', '', '', ''),
			'4Y' => array('4Y', 'Home Health Services', '', '', ''),
			'4Z' => array('4Z', 'Hospice', '', '', ''),
			'41' => array('41', 'Submitter', '', '', ''),
			'5A' => array('5A', 'Medical Surgical or Other Intensive Care Unit', '', '', ''),
			'5B' => array('5B', 'Hisopathology Laboratory', '', '', ''),
			'5C' => array('5C', 'Blood Bank', '', '', ''),
			'5D' => array('5D', 'Neonatal Intensive Care Unit', '', '', ''),
			'5E' => array('5E', 'Obstetrics Unit', '', '', ''),
			'5F' => array('5F', 'Occupational Health Services', '', '', ''),
			'5G' => array('5G', 'Organized Outpatient Services', '', '', ''),
			'5H' => array('5H', 'Pediatric Acute Inpatient Unit', '', '', ''),
			'5I' => array('5I', 'Psychiatric Child/Adolescent Services', '', '', ''),
			'5J' => array('5J', 'Psychiatric Consultation-Liaison Services', '', '', ''),
			'5K' => array('5K', 'Psychiatric Education Services', '', '', ''),
			'5L' => array('5L', 'Psychiatric Emergency Services', '', '', ''),
			'5M' => array('5M', 'Psychiatric Geriatric Services', '', '', ''),
			'5N' => array('5N', 'Psychiatric Inpatient Unit', '', '', ''),
			'5O' => array('5O', 'Psychiatric Outpatient Services', '', '', ''),
			'5P' => array('5P', 'Psychiatric Partial Hospitalization Program', '', '', ''),
			'5Q' => array('5Q', 'Megavoltage Radiation Therapy Unit', '', '', ''),
			'5R' => array('5R', 'Radioactive Implants Unit', '', '', ''),
			'5S' => array('5S', 'Theraputic Radioisotope Facility', '', '', ''),
			'5T' => array('5T', 'X-Ray Radiation Therapy Unit', '', '', ''),
			'5U' => array('5U', 'CT Scanner Unit', '', '', ''),
			'5V' => array('5V', 'Diagnostic Radioisotope Facility', '', '', ''),
			'5W' => array('5W', 'Magnetic Resonance Imaging (MRI) Facility', '', '', ''),
			'5X' => array('5X', 'Ultrasound Unit', '', '', ''),
			'5Y' => array('5Y', 'Rehabilitation Inpatient Unit', '', '', ''),
			'5Z' => array('5Z', 'Rehabilitation Outpatient Services', '', '', ''),
			'61' => array('61', 'Performed At', '', '', ''),
			'6A' => array('6A', 'Reproductive Health Services', '', '', ''),
			'6B' => array('6B', 'Skilled Nursing or Other Long-Term Care Unit', '', '', ''),
			'6C' => array('6C', 'Single Photon Emission Computerized Tomography (SPECT) Unit', '', '', ''),
			'6D' => array('6D', 'Organized Social Work Service Facility', '', '', ''),
			'6E' => array('6E', 'Outpatient Social Work Services', '', '', ''),
			'6F' => array('6F', 'Emergency Department Social Work Services', '', '', ''),
			'6G' => array('6G', 'Sports Medicine Clinic/Services', '', '', ''),
			'6H' => array('6H', 'Hospital Auxiliary Unit', '', '', ''),
			'6I' => array('6I', 'Patient Representative Services', '', '', ''),
			'6J' => array('6J', 'Volunteer Services Department', '', '', ''),
			'6K' => array('6K', 'Outpatient Surgery Services', '', '', ''),
			'6L' => array('6L', 'Organ/Tissue Transplant Unit', '', '', ''),
			'6M' => array('6M', 'Orthopedic Surgery Facility', '', '', ''),
			'6N' => array('6N', 'Occupational Therapy Services', '', '', ''),
			'6O' => array('6O', 'Physical Therapy Services', '', '', ''),
			'6P' => array('6P', 'Recreational Therapy Services', '', '', ''),
			'6Q' => array('6Q', 'Respiratory Therapy Services', '', '', ''),
			'6R' => array('6R', 'Speech Therapy Services', '', '', ''),
			'6S' => array('6S', 'Women´s Health Center/Services', '', '', ''),
			'6U' => array('6U', 'Cardiac Rehabilitation Program Facility', '', '', ''),
			'6V' => array('6V', 'Non-Invasive Cardiac Assessment Services', '', '', ''),
			'6W' => array('6W', 'Emergency Medical Technician', '', '', ''),
			'6X' => array('6X', 'Disciplinary Contact', '', '', ''),
			'6Y' => array('6Y', 'Case Manager', '', '', ''),
			'70' => array('70', 'Prior Incorrect Insured', '', '', ''),
			'71' => array('71', 'Attending Physician', '', '', ''),
			'72' => array('72', 'Operating Physician', '', '', ''),
			'73' => array('73', 'Other Physician', '', '', ''),
			'74' => array('74', 'Corrected Insured', '', '', ''),
			'75' => array('75', 'Participant', '', '', ''),
			'77' => array('77', 'Service Location', '', '', ''),
			'7C' => array('7C', 'Place of Occurrence', '', '', ''),
			'80' => array('80', 'Hospital', '', '', ''),
			'82' => array('82', 'Rendering Provider', '', '', ''),
			'84' => array('84', 'Subscriber´s Employer', '', '', ''),
			'85' => array('85', 'Billing Provider', '', '', ''),
			'87' => array('87', 'Pay-to Provider', '', '', ''),
			'95' => array('95', 'Research Institute', '', '', ''),
			'AY' => array('AY', 'Clearinghouse','', '', ''), 
			'CK' => array('CK', 'Pharmacist', '', '', ''),
			'CZ' => array('CZ', 'Admitting Surgeon', '', '', ''),
			'D2' => array('D2', 'Commercial Insurer', '', '', ''),
			'DD' => array('DD', 'Assistant Surgeon', '', '', ''),
			'DJ' => array('DJ', 'Consulting Physician', '', '', ''),
			'DK' => array('DK', 'Ordering Physician', '', '', ''),
			'DN' => array('DN', 'Referring Provider', '', '', ''),
			'DO' => array('DO', 'Dependent Name', '', '', ''),
			'DQ' => array('DQ', 'Supervising Physician', '', '', ''),
			'E1' => array('E1', 'Person or Other Entity Legally Responsible for a Child', '', '', ''),
			'E2' => array('E2', 'Person or Other Entity With Whom a Child Resides', '', '', ''),
			'E7' => array('E7', 'Previous Employer', '', '', ''),
			'E9' => array('E9', 'Participating Laboratory', '', '', ''),
			'FA' => array('FA', 'Facility', '', '', ''),
			'FD' => array('FD', 'Physical Address', '', '', ''),
			'FE' => array('FE', 'Mail Address', '', '', ''),
			'G0' => array('G0', 'Dependent Insured', '', '', ''),
			'G3' => array('G3', 'Clinic', '', '', ''),
			'GB' => array('GB', 'Other Insured', '', '', ''),
			'GD' => array('GD', 'Guardian', '', '', ''),
			'GI' => array('GI', 'Paramedic', '', '', ''),
			'GJ' => array('GJ', 'Paramedical Company', '', '', ''),
			'GK' => array('GK', 'Previous Insured', '', '', ''),
			'GM' => array('GM', 'Spouse Insured', '', '', ''),
			'GP' => array('GP', 'Gateway Provider', '', '', ''),
			'GW' => array('GW', 'Group', '', '', ''),
			'GY' => array('GY', 'Treatment Facility', '', '', ''),
			'HF' => array('HF', 'Healthcare Professional Shortage Area (HPSA) Facility', '', '', ''),
			'HH' => array('HH', 'Home Health Agency', '', '', ''),
			'I3' => array('I3', 'Independent Physicians Association (IPA)', '', '', ''),
			'IJ' => array('IJ', 'Injection Point', '', '', ''),
			'IL' => array('IL', 'Insured or Subscriber', '', '', ''),
			'IN' => array('IN', 'Insurer', '', '', ''),
			'LI' => array('LI', 'Independent Lab', '', '', ''),
			'LR' => array('LR', 'Legal Representative', '', '', ''),
			'MR' => array('MR', 'Medical Insurance Carrier', '', '', ''),
			'NZ' => array('NZ', 'Primary Physician', '', '', ''),
			'OB' => array('OB', 'Ordered By', '', '', ''),
			'OC' => array('OC', 'Origin Carrier', '', '', ''),
			'OD' => array('OD', 'Doctor of Optometry', '', '', ''),
			'OX' => array('OX', 'Oxygen Therapy Facility', '', '', ''),
			'P0' => array('P0', 'Patient Facility', '', '', ''),
			'P2' => array('P2', 'Primary Insured or Subscriber', '', '', ''),
			'P3' => array('P3', 'Primary Care Provider', '', '', ''),
			'P4' => array('P4', 'Prior Insurance Carrier', '', '', ''),
			'P5' => array('P5', 'Plan Sponsor', '', '', ''),
			'P6' => array('P6', 'Third Party Reviewing Preferred Provider Organization (PPO)', '', '', ''),
			'P7' => array('P7', 'Third Party Repricing Preferred Provider Organization (PPO)', '', '', ''),
			'PR' => array('PR', 'Payer', '', '', ''),
			'PT' => array('PT', 'Party to Receive Test Report', '', '', ''),
			'PV' => array('PV', 'Party performing certification', '', '', ''),
			'PW' => array('PW', 'Pick Up Address', '', '', ''),
			'QA' => array('QA', 'Pharmacy', '', '', ''),
			'QB' => array('QB', 'Purchase Service Provider', '', '', ''),
			'QC' => array('QC', 'Patient', '', '', ''),
			'QD' => array('QD', 'Responsible Party', '', '', ''),
			'QE' => array('QE', 'Policyholder', '', '', ''),
			'QH' => array('QH', 'Physician', '', '', ''),
			'QK' => array('QK', 'Managed Care', '', '', ''),
			'QL' => array('QL', 'Chiropractor', '', '', ''),
			'QN' => array('QN', 'Dentist', '', '', ''),
			'QO' => array('QO', 'Doctor of Osteopathy', '', '', ''),
			'QS' => array('QS', 'Podiatrist', '', '', ''),
			'QV' => array('QV', 'Group Practice', '', '', ''),
			'QY' => array('QY', 'Medical Doctor', '', '', ''),
			'RC' => array('RC', 'Receiving Location', '', '', ''),
			'RW' => array('RW', 'Rural Health Clinic', '', '', ''),
			'S4' => array('S4', 'Skilled Nursing Facility', '', '', ''),
			'SJ' => array('SJ', 'Service Provider', '', '', ''),
			'SU' => array('SU', 'Supplier/Manufacturer', '', '', ''),
			'T4' => array('T4', 'Transfer Point', '', '', ''),
			'TQ' => array('TQ', 'Third Party Reviewing Organization (TPO)', '', '', ''),
			'TT' => array('TT', 'Transfer To', '', '', ''),
			'TU' => array('TU', 'Third Party Repricing Organization (TPO)', '', '', ''),
			'UH' => array('UH', 'Nursing Home', '', '', ''),
			'VN' => array('VN', 'Vendor', '', '', ''),
			'X3' => array('X3', 'Utilization Management Organization', '', '', ''),
			'X4' => array('X4', 'Spouse', '', '', ''),
			'X5' => array('X5', 'Durable Medical Equipment Supplier', '', '', ''),
			'Y2' => array('Y2', 'Managed Care Organization', '', '', ''),
			'ZZ' => array('ZZ', 'Mutually Defined', '', '', ''),
			'MSC' => array('MSC', 'Mammography Screening Center', '', '', ''),
			'PRP' => array('PRP', 'Primary Payer', '', '', ''),
			'SEP' => array('SEP', 'Secondary Payer', '', '', ''),
			'TL' => array('TL', 'Testing Laboratory', '', '', ''),
			'TTP' => array('TTP', 'Tertiary Payer', '', '', ''),
			'EXS' => array('EXS', 'Ex-spouse', '', '', ''),		
			'NZ' => array('NZ', 'Primary Physician', '', '', ''),
			'AAG' => array('AAG', 'Ground Ambulance Services', '', '', ''),
			'AAK' => array('AAK', 'Primary Surgeon', '', '', ''),
			'AAL' => array('AAL', 'Medical Nurse', '', '', ''),
			'AAM' => array('AAM', 'Cardiac Rehabilitation Services', '', '', ''),
			'AAN' => array('AAN', 'Skilled Nursing Services', '', '', ''),
			'AAO' => array('AAO', 'Observation Room Services', '', '', ''),
			'AAQ' => array('AAQ', 'Anesthesiology Services', '', '', ''),
			'NCT' => array('NCT', 'Name Changed To', '', '', ''),
			'ORI' => array('ORI', 'Original Name', '', '', ''),
			'VER' => array('VER', 'Party Performing Verification', '', '', ''),	
			);
			if (array_key_exists($code, $this->STC_Entity_Code)){
				return $this->STC_Entity_Code[$code];
			} else {
				return array($code, "Not found in STC Entity codes table", "", "", "");
			}
		}
	} // end get_STC_Entity_Code

// 
	/**
	 * entity identifier code  --code source 237
	 * 
	 * Codes from http://www.wpc-edi.com 
	 * Claim Status Category Codes LAST UPDATED 7/1/2012
	 * 
	 * @param string code
	 * @return array 0=>code, [1]=>description, [2]=>empty, [3]=>empty  [4]=>empty
	 */	 
	public function get_STC_Category_Code( $code ) {
		// 0 => code, [1] => description, [2] => dates, [3] => notes  (empty, none available)
		// contains "Unknown" for codes for which I did not find definitions
		if ( is_array( $this->STC_Category_Code )) {
			if (array_key_exists($code, $this->STC_Category_Code)){
				return $this->STC_Category_Code[$code];
			} else {
				return array($code, "Not found in STC Category codes table", "", "", "");
			}
		} else {
			$this->STC_Category_Code = array(	
			'A0' =>  array('A0', 'Acknowledgement/Forwarded-The claim/encounter has been forwarded to another entity.', '', '', ''),
			'A1' =>  array('A1', 'Acknowledgement/Receipt-The claim/encounter has been received.', '', '', ''),
			'A2' =>  array('A2', 'Acknowledgement/Acceptance into adjudication system-The claim/encounter has been accepted into the adjudication system.', '', '', ''),
			'A3' =>  array('A3', 'Acknowledgement/Returned as unprocessable claim-The claim/encounter has been rejected and has not been entered into the adjudication system.', '', '', ''),
			'A4' =>  array('A4', 'Acknowledgement/Not Found-The claim/encounter can not be found in the adjudication system.', '', '', ''),
			'A5' =>  array('A5', 'Acknowledgement/Split Claim-The claim/encounter has been split upon acceptance into the adjudication system.', '', '', ''),
			'A6' =>  array('A6', 'Acknowledgement/Rejected for Missing Information - The claim/encounter is missing the information specified in the Status details and has been rejected.', '', '', ''),
			'A7' =>  array('A7', 'Acknowledgement/Rejected for Invalid Information - The claim/encounter has invalid information as specified in the Status details and has been rejected.', '', '', ''),
			'A8' =>  array('A8', 'Acknowledgement/Rejected for relational field in error.', '', '', ''),
			// Pending
			'P0' =>  array('P0', 'Pending: Adjudication/Details-This is a generic message about a pended claim.', '', '', ''),
			'P1' =>  array('P1', 'Pending/In Process-The claim or encounter is in the adjudication system.', '', '', ''),
			'P2' =>  array('P2', 'Pending/Payer Review-The claim/encounter is suspended and is pending review ', '', '', ''),
			'P3' =>  array('P3', 'Pending/Provider Requested Information - The claim or encounter is waiting for information that has already been requested from the provider.', '', '', ''),
			'P4' =>  array('P4', 'Pending/Patient Requested Information - The claim or encounter is waiting for information that has already been requested from the patient.', '', '', ''),
			'P5' =>  array('P5', 'Pending/Payer Administrative/System hold', '', '', ''),
			// Finalized
			'F0' =>  array('F0', 'Finalized-The claim/encounter has completed the adjudication cycle and no more action will be taken.', '', '', ''),
			'F1' =>  array('F1', 'Finalized/Payment-The claim/line has been paid.', '', '', ''),
			'F2' =>  array('F2', 'Finalized/Denial-The claim/line has been denied.', '', '', ''),
			'F3' =>  array('F3', 'Finalized/Revised - Adjudication information has been changed', '', '', ''),
			'F3F' =>  array('F3F', 'Finalized/Forwarded-The claim/encounter processing has been completed.', '', '', ''),
			'F3N' =>  array('F3N', 'Finalized/Not Forwarded-The claim/encounter processing has been completed.', '', '', ''),
			'F4' =>  array('F4', 'Finalized/Adjudication Complete - No payment forthcoming-The claim/encounter has been adjudicated and no further payment is forthcoming.', '', '', ''),
			// Requests for additional information
			'R0' =>  array('R0', 'Requests for additional Information/General Requests-Requests that don\'t fall into other R-type categories.', '', '', ''),
			'R1' =>  array('R1', 'Requests for additional Information/Entity Requests-Requests for information about specific entities ', '', '', ''),
			'R3' =>  array('R3', 'Requests for additional Information/Claim/Line-Requests for information that could normally be submitted on a claim.', '', '', ''),
			'R4' =>  array('R4', 'Requests for additional Information/Documentation-Requests for additional supporting documentation.', '', '', ''),
			'R5' =>  array('R5', 'Request for additional information/more specific detail-Additional information as a follow up to a previous request is needed.', '', '', ''),
			'R6' =>  array('R6', 'Requests for additional information – Regulatory requirements', '', '', ''),
			'R7' =>  array('R7', 'Requests for additional information – Confirm care is consistent with Health Plan policy coverage', '', '', ''),
			'R8' =>  array('R8', 'Requests for additional information – Confirm care is consistent with health plan coverage exceptions', '', '', ''),
			'R9' =>  array('R9', 'Requests for additional information – Determination of medical necessity', '', '', ''),
			'R10' =>  array('R10', 'Requests for additional information – Support a filed grievance or appeal', '', '', ''),
			'R11' =>  array('R11', 'Requests for additional information – Pre-payment review of claims', '', '', ''),
			'R12' =>  array('R12', 'Requests for additional information – Clarification or justification of use for specified procedure code', '', '', ''),
			'R13' =>  array('R13', 'Requests for additional information – Original documents submitted are not readable.', '', '', ''),
			'R14' =>  array('R14', 'Requests for additional information – Original documents received are not what was requested.', '', '', ''),
			'R15' =>  array('R15', 'Requests for additional information – Workers Compensation coverage determination.', '', '', ''),
			'R16' =>  array('R16', 'Requests for additional information – Eligibility determination', '', '', ''),
			// General
			// Error
			'E0' =>  array('E0', 'Response not possible - error on submitted request data', '', '', ''),
			'E1' =>  array('E1', 'Response not possible - System Status', '', '', ''),
			'E2' =>  array('E2', 'Information Holder is not responding; resubmit at a later time.', '', '', ''),
			'E3' =>  array('E3', 'Correction required - relational fields in error.', '', '', ''),
			'E4' =>  array('E4', 'Trading partner agreement specific requirement not met: Data correction required.', '', '', ''),
			'Searches' =>  array('Searches', '', '', '', ''),
			'D0' =>  array('D0', 'Data Search Unsuccessful - The payer is unable to return status on the requested claim(s) based on the submitted search criteria.', '', '', ''),
			);
			if (array_key_exists($code, $this->STC_Category_Code)){
				return $this->STC_Category_Code[$code];
			} else {
				return array($code, "Not found in STC Category codes table", "", "", "");
			}
		}
	} // end get_STC_Category_Code

	/**
	 * Health Care Services Decision Reason Codes LAST UPDATED 7/1/2012
	 * 
	 * @param string code
	 * @return array 0=>code, [1]=>description, [2]=>empty, [3]=>empty  [4]=>empty
	 */
	public function get_STC_HCSDR_Code( $code ) {
		// 0 => code, [1] => description, [2] => dates, [3] => notes  (empty, none available)
		// contains "Unknown" for codes for which I did not find definitions
		if ( is_array( $this->STC_HCSDR_Code )) {
			if (array_key_exists($code, $this->STC_HCSDR_Code)){
				return $this->STC_HCSDR_Code[$code];
			} else {
				return array($code, "Not found in STC_HCSDR_Code codes table", "", "", "");
			}
		} else {
			$this->STC_HCSDR_Code = array(		
			'1' =>  array('1', 'Price Authorization Expired', '', '', ''),
			'2' =>  array('2', 'Price authorization no longer required', '', '', ''),
			'3' =>  array('3', 'Product not on the price authorization', '', '', ''),
			'4' =>  array('4', 'Authorized Quantity Exceeded', '', '', ''),
			'5' =>  array('5', 'Special Cost Incorrect', '', '', ''),
			'6' =>  array('6', 'No Credit Allowed', '', '', ''),
			'7' =>  array('7', 'Administrative Cancellation', '', '', ''),
			'8' =>  array('8', 'Unit resale higher than authorized', '', '', ''),
			'9' =>  array('9', 'Out of Network', '', '', ''),
			'0A' =>  array('0A', 'Testing not Included', '', '', ''),
			'0B' =>  array('0B', 'Request Forwarded To and Decision Response Forthcoming From an External Review Organization', '', '', ''),
			'0C' =>  array('0C', 'Authorization/Access Restrictions', '', '', ''),
			'0D' =>  array('0D', 'Requires PCP authorization', '', '', ''),
			'0E' =>  array('0E', 'Provider is Not Primary Care Physician', '', '', ''),
			'0F' =>  array('0F', 'Not Medically Necessary', '', '', ''),
			'0G' =>  array('0G', 'Level of Care Not Appropriate', '', '', ''),
			'0H' =>  array('0H', 'Certification Not Required for this Service', '', '', ''),
			'0J' =>  array('0J', 'Certification Responsibility of External Review Organization', '', '', ''),
			'0K' =>  array('0K', 'Primary Care Service', '', '', ''),
			'0L' =>  array('0L', 'Exceeds Plan Maximums', '', '', ''),
			'0M' =>  array('0M', 'Non-covered Service', '', '', ''),
			'0N' =>  array('0N', 'No Prior Approval', '', '', ''),
			'0P' =>  array('0P', 'Requested Information Not Received', '', '', ''),
			'0Q' =>  array('0Q', 'Duplicate Request', '', '', ''),
			'0R' =>  array('0R', 'Service Inconsistent with Diagnosis', '', '', ''),
			'0S' =>  array('0S', 'Pre-existing Condition', '', '', ''),
			'0T' =>  array('0T', 'Experimental Service or Procedure', '', '', ''),
			'0U' =>  array('0U', 'Additional Patient Information required', '', '', ''),
			'0V' =>  array('0V', 'Requires Medical Review', '', '', ''),
			'0W' =>  array('0W', 'Disposition pending review', '', '', ''),
			'0X' =>  array('0X', 'Service Inconsistent with Provider Type', '', '', ''),
			'0Y' =>  array('0Y', 'Service inconsistent with Patient\'s Age', '', '', ''),
			'0Z' =>  array('0Z', 'Service inconsistent with Patient\'s Gender', '', '', ''),
			'10' =>  array('10', 'Product/service/procedure delivery pattern ', '', '', ''),
			'11' =>  array('11', 'Pricing', '', '', ''),
			'12' =>  array('12', 'Patient is restricted to specific provider', '', '', ''),
			'13' =>  array('13', 'Service authorized for another provider', '', '', ''),
			'14' =>  array('14', 'Plan/contractual guidelines not followed', '', '', ''),
			'15' =>  array('15', 'Plan/contractual geographic restriction', '', '', ''),
			'16' =>  array('16', 'Inappropriate facility type', '', '', ''),
			'17' =>  array('17', 'Time limits not met', '', '', ''),
			'18' =>  array('18', 'Notification received', '', '', ''),
			'19' =>  array('19', 'Cosmetic', '', '', ''),
			'20' =>  array('20', 'Once in a lifetime restriction applies', '', '', ''),
			'21' =>  array('21', 'Transport Request Denied', '', '', ''),
			'22' =>  array('22', 'Ambulance Certification Segment information doesn\'t correspond to Transport Address Segment', '', '', ''),
			'23' =>  array('23', 'Mileage cannot be computed based on data submitted', '', '', ''),
			'24' =>  array('24', 'Computed mileage is inconsistent with transport information or service units submitted', '', '', ''),
			'25' =>  array('25', 'Services were not considered due to other errors in the request.', '', '', ''),
			'26' =>  array('26', 'Missing Provider Role', '', '', ''),
			);
			if (array_key_exists($code, $this->STC_HCSDR_Code)){
				return $this->STC_HCSDR_Code[$code];
			} else {
				return array($code, "Not found in Health Care Service Decision Rules codes table", "", "", "");
			}
		}
	} // end get_STC_HCSDR_Code


	/**
	 * Health Care Claim Status Codes LAST UPDATED 7/1/2012
	 * http://www.wpc-edi.com/reference/codelists/healthcare/claim-status-codes/
	 * 
	 * @param string code
	 * @return array 0=>code, [1]=>description, [2]=>empty, [3]=>empty  [4]=>empty
	 */
	public function get_STC_Status_Code( $code ) {
	// 0 => code, [1] => description, [2] => dates, [3] => notes  (empty, none available)
	// contains "Unknown" for codes for which I did not find definitions
	if ( is_array( $this->STC_Status_Code )) {
		if (array_key_exists($code, $this->STC_Status_Code)){
			return $this->STC_Status_Code[$code];
		} else {
			return array($code, "Not found in STC Status codes table", "", "", "");
		}
	} else {
		$this->STC_Status_Code = array(
			'0' =>  array('0', 'Cannot provide further status electronically.', '', '', ''),
			'1' =>  array('1', 'For more detailed information, see remittance advice.', '', '', ''),
			'2' =>  array('2', 'More detailed information in letter.', '', '', ''),
			'3' =>  array('3', 'Claim has been adjudicated and is awaiting payment cycle.', '', '', ''),
			'6' =>  array('6', 'Balance due from the subscriber.', '', '', ''),
			'12' =>  array('12', 'One or more originally submitted procedure codes have been combined.', '', '', ''),
			'15' =>  array('15', 'One or more originally submitted procedure code have been modified.', '', '', ''),
			'16' =>  array('16', 'Claim/encounter has been forwarded to entity.', '', '', ''),
			'17' =>  array('17', 'Claim/encounter has been forwarded by third party entity to entity.', '', '', ''),
			'18' =>  array('18', 'Entity received claim/encounter, but returned invalid status.', '', '', ''),
			'19' =>  array('19', 'Entity acknowledges receipt of claim/encounter.', '', '', ''),
			'20' =>  array('20', 'Accepted for processing.', '', '', ''),
			'21' =>  array('21', 'Missing or invalid information.', '', '', ''),
			'23' =>  array('23', 'Returned to Entity.', '', '', ''),
			'24' =>  array('24', 'Entity not approved as an electronic submitter.', '', '', ''),
			'25' =>  array('25', 'Entity not approved.', '', '', ''),
			'26' =>  array('26', 'Entity not found.', '', '', ''),
			'27' =>  array('27', 'Policy canceled.', '', '', ''),
			'29' =>  array('29', 'Subscriber and policy number/contract number mismatched.', '', '', ''),
			'30' =>  array('30', 'Subscriber and subscriber id mismatched.', '', '', ''),
			'31' =>  array('31', 'Subscriber and policyholder name mismatched.', '', '', ''),
			'32' =>  array('32', 'Subscriber and policy number/contract number not found.', '', '', ''),
			'33' =>  array('33', 'Subscriber and subscriber id not found.', '', '', ''),
			'34' =>  array('34', 'Subscriber and policyholder name not found.', '', '', ''),
			'35' =>  array('35', 'Claim/encounter not found.', '', '', ''),
			'37' =>  array('37', 'Predetermination is on file, awaiting completion of services.', '', '', ''),
			'38' =>  array('38', 'Awaiting next periodic adjudication cycle.', '', '', ''),
			'39' =>  array('39', 'Charges for pregnancy deferred until delivery.', '', '', ''),
			'40' =>  array('40', 'Waiting for final approval.', '', '', ''),
			'41' =>  array('41', 'Special handling required at payer site.', '', '', ''),
			'42' =>  array('42', 'Awaiting related charges.', '', '', ''),
			'44' =>  array('44', 'Charges pending provider audit.', '', '', ''),
			'45' =>  array('45', 'Awaiting benefit determination.', '', '', ''),
			'46' =>  array('46', 'Internal review/audit.', '', '', ''),
			'47' =>  array('47', 'Internal review/audit - partial payment made.', '', '', ''),
			'49' =>  array('49', 'Pending provider accreditation review.', '', '', ''),
			'50' =>  array('50', 'Claim waiting for internal provider verification.', '', '', ''),
			'51' =>  array('51', 'Investigating occupational illness/accident.', '', '', ''),
			'52' =>  array('52', 'Investigating existence of other insurance coverage.', '', '', ''),
			'53' =>  array('53', 'Claim being researched for Insured ID/Group Policy Number error.', '', '', ''),
			'54' =>  array('54', 'Duplicate of a previously processed claim/line.', '', '', ''),
			'55' =>  array('55', 'Claim assigned to an approver/analyst.', '', '', ''),
			'56' =>  array('56', 'Awaiting eligibility determination.', '', '', ''),
			'57' =>  array('57', 'Pending COBRA information requested.', '', '', ''),
			'59' =>  array('59', 'Information was requested by a non-electronic method.', '', '', ''),
			'60' =>  array('60', 'Information was requested by an electronic method.', '', '', ''),
			'61' =>  array('61', 'Eligibility for extended benefits.', '', '', ''),
			'64' =>  array('64', 'Re-pricing information.', '', '', ''),
			'65' =>  array('65', 'Claim/line has been paid.', '', '', ''),
			'66' =>  array('66', 'Payment reflects usual and customary charges.', '', '', ''),
			'72' =>  array('72', 'Claim contains split payment.', '', '', ''),
			'73' =>  array('73', 'Payment made to entity, assignment of benefits not on file.', '', '', ''),
			'78' =>  array('78', 'Duplicate of an existing claim/line, awaiting processing.', '', '', ''),
			'81' =>  array('81', 'Contract/plan does not cover pre-existing conditions.', '', '', ''),
			'83' =>  array('83', 'No coverage for newborns.', '', '', ''),
			'84' =>  array('84', 'Service not authorized.', '', '', ''),
			'85' =>  array('85', 'Entity not primary.', '', '', ''),
			'86' =>  array('86', 'Diagnosis and patient gender mismatch.', '', '', ''),
			'88' =>  array('88', 'Entity not eligible for benefits for submitted dates of service.', '', '', ''),
			'89' =>  array('89', 'Entity not eligible for dental benefits for submitted dates of service.', '', '', ''),
			'90' =>  array('90', 'Entity not eligible for medical benefits for submitted dates of service.', '', '', ''),
			'91' =>  array('91', 'Entity not eligible/not approved for dates of service.', '', '', ''),
			'92' =>  array('92', 'Entity does not meet dependent or student qualification.', '', '', ''),
			'93' =>  array('93', 'Entity is not selected primary care provider.', '', '', ''),
			'94' =>  array('94', 'Entity not referred by selected primary care provider.', '', '', ''),
			'95' =>  array('95', 'Requested additional information not received.', '', '', ''),
			'96' =>  array('96', 'No agreement with entity.', '', '', ''),
			'97' =>  array('97', 'Patient eligibility not found with entity.', '', '', ''),
			'98' =>  array('98', 'Charges applied to deductible.', '', '', ''),
			'99' =>  array('99', 'Pre-treatment review.', '', '', ''),
			'100' =>  array('100', 'Pre-certification penalty taken.', '', '', ''),
			'101' =>  array('101', 'Claim was processed as adjustment to previous claim.', '', '', ''),
			'102' =>  array('102', 'Newborn\'s charges processed on mother\'s claim.', '', '', ''),
			'103' =>  array('103', 'Claim combined with other claim(s).', '', '', ''),
			'104' =>  array('104', 'Processed according to plan provisions ', '', '', ''),
			'105' =>  array('105', 'Claim/line is capitated.', '', '', ''),
			'106' =>  array('106', 'This amount is not entity\'s responsibility.', '', '', ''),
			'107' =>  array('107', 'Processed according to contract provisions ', '', '', ''),
			'109' =>  array('109', 'Entity not eligible.', '', '', ''),
			'110' =>  array('110', 'Claim requires pricing information.', '', '', ''),
			'111' =>  array('111', 'At the policyholder\'s request these claims cannot be submitted electronically.', '', '', ''),
			'114' =>  array('114', 'Claim/service should be processed by entity.', '', '', ''),
			'116' =>  array('116', 'Claim submitted to incorrect payer.', '', '', ''),
			'117' =>  array('117', 'Claim requires signature-on-file indicator.', '', '', ''),
			'121' =>  array('121', 'Service line number greater than maximum allowable for payer.', '', '', ''),
			'123' =>  array('123', 'Additional information requested from entity.', '', '', ''),
			'124' =>  array('124', 'Entity\'s name, address, phone and id number.', '', '', ''),
			'125' =>  array('125', 'Entity\'s name.', '', '', ''),
			'126' =>  array('126', 'Entity\'s address.', '', '', ''),
			'127' =>  array('127', 'Entity\'s Communication Number.', '', '', ''),
			'128' =>  array('128', 'Entity\'s tax id.', '', '', ''),
			'129' =>  array('129', 'Entity\'s Blue Cross provider id.', '', '', ''),
			'130' =>  array('130', 'Entity\'s Blue Shield provider id.', '', '', ''),
			'131' =>  array('131', 'Entity\'s Medicare provider id.', '', '', ''),
			'132' =>  array('132', 'Entity\'s Medicaid provider id.', '', '', ''),
			'133' =>  array('133', 'Entity\'s UPIN.', '', '', ''),
			'134' =>  array('134', 'Entity\'s CHAMPUS provider id.', '', '', ''),
			'135' =>  array('135', 'Entity\'s commercial provider id.', '', '', ''),
			'136' =>  array('136', 'Entity\'s health industry id number.', '', '', ''),
			'137' =>  array('137', 'Entity\'s plan network id.', '', '', ''),
			'138' =>  array('138', 'Entity\'s site id .', '', '', ''),
			'139' =>  array('139', 'Entity\'s health maintenance provider id ', '', '', ''),
			'140' =>  array('140', 'Entity\'s preferred provider organization id ', '', '', ''),
			'141' =>  array('141', 'Entity\'s administrative services organization id ', '', '', ''),
			'142' =>  array('142', 'Entity\'s license/certification number.', '', '', ''),
			'143' =>  array('143', 'Entity\'s state license number.', '', '', ''),
			'144' =>  array('144', 'Entity\'s specialty license number.', '', '', ''),
			'145' =>  array('145', 'Entity\'s specialty/taxonomy code.', '', '', ''),
			'146' =>  array('146', 'Entity\'s anesthesia license number.', '', '', ''),
			'147' =>  array('147', 'Entity\'s qualification degree/designation ', '', '', ''),
			'148' =>  array('148', 'Entity\'s social security number.', '', '', ''),
			'149' =>  array('149', 'Entity\'s employer id.', '', '', ''),
			'150' =>  array('150', 'Entity\'s drug enforcement agency ', '', '', ''),
			'152' =>  array('152', 'Pharmacy processor number.', '', '', ''),
			'153' =>  array('153', 'Entity\'s id number.', '', '', ''),
			'154' =>  array('154', 'Relationship of surgeon & assistant surgeon.', '', '', ''),
			'155' =>  array('155', 'Entity\'s relationship to patient.', '', '', ''),
			'156' =>  array('156', 'Patient relationship to subscriber', '', '', ''),
			'157' =>  array('157', 'Entity\'s Gender.', '', '', ''),
			'158' =>  array('158', 'Entity\'s date of birth.', '', '', ''),
			'159' =>  array('159', 'Entity\'s date of death.', '', '', ''),
			'160' =>  array('160', 'Entity\'s marital status.', '', '', ''),
			'161' =>  array('161', 'Entity\'s employment status.', '', '', ''),
			'162' =>  array('162', 'Entity\'s health insurance claim number ', '', '', ''),
			'163' =>  array('163', 'Entity\'s policy number.', '', '', ''),
			'164' =>  array('164', 'Entity\'s contract/member number.', '', '', ''),
			'165' =>  array('165', 'Entity\'s employer name, address and phone.', '', '', ''),
			'166' =>  array('166', 'Entity\'s employer name.', '', '', ''),
			'167' =>  array('167', 'Entity\'s employer address.', '', '', ''),
			'168' =>  array('168', 'Entity\'s employer phone number.', '', '', ''),
			'170' =>  array('170', 'Entity\'s employee id.', '', '', ''),
			'171' =>  array('171', 'Other insurance coverage information ', '', '', ''),
			'172' =>  array('172', 'Other employer name, address and telephone number.', '', '', ''),
			'173' =>  array('173', 'Entity\'s name, address, phone, gender, DOB, marital status, employment status and relation to subscriber.', '', '', ''),
			'174' =>  array('174', 'Entity\'s student status.', '', '', ''),
			'175' =>  array('175', 'Entity\'s school name.', '', '', ''),
			'176' =>  array('176', 'Entity\'s school address.', '', '', ''),
			'177' =>  array('177', 'Transplant recipient\'s name, date of birth, gender, relationship to insured.', '', '', ''),
			'178' =>  array('178', 'Submitted charges.', '', '', ''),
			'179' =>  array('179', 'Outside lab charges.', '', '', ''),
			'180' =>  array('180', 'Hospital s semi-private room rate.', '', '', ''),
			'181' =>  array('181', 'Hospital s room rate.', '', '', ''),
			'182' =>  array('182', 'Allowable/paid from other entities coverage NOTE: This code requires the use of an entity code.', '', '', ''),
			'183' =>  array('183', 'Amount entity has paid.', '', '', ''),
			'184' =>  array('184', 'Purchase price for the rented durable medical equipment.', '', '', ''),
			'185' =>  array('185', 'Rental price for durable medical equipment.', '', '', ''),
			'186' =>  array('186', 'Purchase and rental price of durable medical equipment.', '', '', ''),
			'187' =>  array('187', 'Date(s) of service.', '', '', ''),
			'188' =>  array('188', 'Statement from-through dates.', '', '', ''),
			'189' =>  array('189', 'Facility admission date', '', '', ''),
			'190' =>  array('190', 'Facility discharge date', '', '', ''),
			'191' =>  array('191', 'Date of Last Menstrual Period ', '', '', ''),
			'192' =>  array('192', 'Date of first service for current series/symptom/illness.', '', '', ''),
			'193' =>  array('193', 'First consultation/evaluation date.', '', '', ''),
			'194' =>  array('194', 'Confinement dates.', '', '', ''),
			'195' =>  array('195', 'Unable to work dates/Disability Dates.', '', '', ''),
			'196' =>  array('196', 'Return to work dates.', '', '', ''),
			'197' =>  array('197', 'Effective coverage date(s).', '', '', ''),
			'198' =>  array('198', 'Medicare effective date.', '', '', ''),
			'199' =>  array('199', 'Date of conception and expected date of delivery.', '', '', ''),
			'200' =>  array('200', 'Date of equipment return.', '', '', ''),
			'201' =>  array('201', 'Date of dental appliance prior placement.', '', '', ''),
			'202' =>  array('202', 'Date of dental prior replacement/reason for replacement.', '', '', ''),
			'203' =>  array('203', 'Date of dental appliance placed.', '', '', ''),
			'204' =>  array('204', 'Date dental canal(s) opened and date service completed.', '', '', ''),
			'205' =>  array('205', 'Date(s) dental root canal therapy previously performed.', '', '', ''),
			'206' =>  array('206', 'Most recent date of curettage, root planing, or periodontal surgery.', '', '', ''),
			'207' =>  array('207', 'Dental impression and seating date.', '', '', ''),
			'208' =>  array('208', 'Most recent date pacemaker was implanted.', '', '', ''),
			'209' =>  array('209', 'Most recent pacemaker battery change date.', '', '', ''),
			'210' =>  array('210', 'Date of the last x-ray.', '', '', ''),
			'211' =>  array('211', 'Date(s) of dialysis training provided to patient.', '', '', ''),
			'212' =>  array('212', 'Date of last routine dialysis.', '', '', ''),
			'213' =>  array('213', 'Date of first routine dialysis.', '', '', ''),
			'214' =>  array('214', 'Original date of prescription/orders/referral.', '', '', ''),
			'215' =>  array('215', 'Date of tooth extraction/evolution.', '', '', ''),
			'216' =>  array('216', 'Drug information.', '', '', ''),
			'217' =>  array('217', 'Drug name, strength and dosage form.', '', '', ''),
			'218' =>  array('218', 'NDC number.', '', '', ''),
			'219' =>  array('219', 'Prescription number.', '', '', ''),
			'222' =>  array('222', 'Drug dispensing units and average wholesale price ', '', '', ''),
			'223' =>  array('223', 'Route of drug/myelogram administration.', '', '', ''),
			'224' =>  array('224', 'Anatomical location for joint injection.', '', '', ''),
			'225' =>  array('225', 'Anatomical location.', '', '', ''),
			'226' =>  array('226', 'Joint injection site.', '', '', ''),
			'227' =>  array('227', 'Hospital information.', '', '', ''),
			'228' =>  array('228', 'Type of bill for UB claim', '', '', ''),
			'229' =>  array('229', 'Hospital admission source.', '', '', ''),
			'230' =>  array('230', 'Hospital admission hour.', '', '', ''),
			'231' =>  array('231', 'Hospital admission type.', '', '', ''),
			'232' =>  array('232', 'Admitting diagnosis.', '', '', ''),
			'233' =>  array('233', 'Hospital discharge hour.', '', '', ''),
			'234' =>  array('234', 'Patient discharge status.', '', '', ''),
			'235' =>  array('235', 'Units of blood furnished.', '', '', ''),
			'236' =>  array('236', 'Units of blood replaced.', '', '', ''),
			'237' =>  array('237', 'Units of deductible blood.', '', '', ''),
			'238' =>  array('238', 'Separate claim for mother/baby charges.', '', '', ''),
			'239' =>  array('239', 'Dental information.', '', '', ''),
			'240' =>  array('240', 'Tooth surface(s) involved.', '', '', ''),
			'241' =>  array('241', 'List of all missing teeth ', '', '', ''),
			'242' =>  array('242', 'Tooth numbers, surfaces, and/or quadrants involved.', '', '', ''),
			'243' =>  array('243', 'Months of dental treatment remaining.', '', '', ''),
			'244' =>  array('244', 'Tooth number or letter.', '', '', ''),
			'245' =>  array('245', 'Dental quadrant/arch.', '', '', ''),
			'246' =>  array('246', 'Total orthodontic service fee, initial appliance fee, monthly fee, length of service.', '', '', ''),
			'247' =>  array('247', 'Line information.', '', '', ''),
			'249' =>  array('249', 'Place of service.', '', '', ''),
			'250' =>  array('250', 'Type of service.', '', '', ''),
			'251' =>  array('251', 'Total anesthesia minutes.', '', '', ''),
			'252' =>  array('252', 'Entity\'s authorization/certification number.', '', '', ''),
			'254' =>  array('254', 'Principal diagnosis code.', '', '', ''),
			'255' =>  array('255', 'Diagnosis code.', '', '', ''),
			'256' =>  array('256', 'DRG code(s).', '', '', ''),
			'257' =>  array('257', 'ADSM-III-R code for services rendered.', '', '', ''),
			'258' =>  array('258', 'Days/units for procedure/revenue code.', '', '', ''),
			'259' =>  array('259', 'Frequency of service.', '', '', ''),
			'260' =>  array('260', 'Length of medical necessity, including begin date.', '', '', ''),
			'261' =>  array('261', 'Obesity measurements.', '', '', ''),
			'262' =>  array('262', 'Type of surgery/service for which anesthesia was administered.', '', '', ''),
			'263' =>  array('263', 'Length of time for services rendered.', '', '', ''),
			'264' =>  array('264', 'Number of liters/minute & total hours/day for respiratory support.', '', '', ''),
			'265' =>  array('265', 'Number of lesions excised.', '', '', ''),
			'266' =>  array('266', 'Facility point of origin and destination - ambulance.', '', '', ''),
			'267' =>  array('267', 'Number of miles patient was transported.', '', '', ''),
			'268' =>  array('268', 'Location of durable medical equipment use.', '', '', ''),
			'269' =>  array('269', 'Length/size of laceration/tumor.', '', '', ''),
			'270' =>  array('270', 'Subluxation location.', '', '', ''),
			'271' =>  array('271', 'Number of spine segments.', '', '', ''),
			'272' =>  array('272', 'Oxygen contents for oxygen system rental.', '', '', ''),
			'273' =>  array('273', 'Weight.', '', '', ''),
			'274' =>  array('274', 'Height.', '', '', ''),
			'275' =>  array('275', 'Claim.', '', '', ''),
			'276' =>  array('276', 'UB04/HCFA-1450/1500 claim form', '', '', ''),
			'277' =>  array('277', 'Paper claim.', '', '', ''),
			'279' =>  array('279', 'Claim/service must be itemized', '', '', ''),
			'281' =>  array('281', 'Related confinement claim.', '', '', ''),
			'282' =>  array('282', 'Copy of prescription.', '', '', ''),
			'283' =>  array('283', 'Medicare entitlement information is required to determine primary coverage', '', '', ''),
			'284' =>  array('284', 'Copy of Medicare ID card.', '', '', ''),
			'286' =>  array('286', 'Other payer\'s Explanation of Benefits/payment information.', '', '', ''),
			'287' =>  array('287', 'Medical necessity for service.', '', '', ''),
			'288' =>  array('288', 'Hospital late charges', '', '', ''),
			'290' =>  array('290', 'Pre-existing information.', '', '', ''),
			'291' =>  array('291', 'Reason for termination of pregnancy.', '', '', ''),
			'292' =>  array('292', 'Purpose of family conference/therapy.', '', '', ''),
			'293' =>  array('293', 'Reason for physical therapy.', '', '', ''),
			'294' =>  array('294', 'Supporting documentation.', '', '', ''),
			'295' =>  array('295', 'Attending physician report.', '', '', ''),
			'296' =>  array('296', 'Nurse\'s notes.', '', '', ''),
			'297' =>  array('297', 'Medical notes/report.', '', '', ''),
			'298' =>  array('298', 'Operative report.', '', '', ''),
			'299' =>  array('299', 'Emergency room notes/report.', '', '', ''),
			'300' =>  array('300', 'Lab/test report/notes/results.', '', '', ''),
			'301' =>  array('301', 'MRI report.', '', '', ''),
			'305' =>  array('305', 'Radiology/x-ray reports and/or interpretation', '', '', ''),
			'306' =>  array('306', 'Detailed description of service.', '', '', ''),
			'307' =>  array('307', 'Narrative with pocket depth chart.', '', '', ''),
			'308' =>  array('308', 'Discharge summary.', '', '', ''),
			'310' =>  array('310', 'Progress notes for the six months prior to statement date.', '', '', ''),
			'311' =>  array('311', 'Pathology notes/report.', '', '', ''),
			'312' =>  array('312', 'Dental charting.', '', '', ''),
			'313' =>  array('313', 'Bridgework information.', '', '', ''),
			'314' =>  array('314', 'Dental records for this service.', '', '', ''),
			'315' =>  array('315', 'Past perio treatment history.', '', '', ''),
			'316' =>  array('316', 'Complete medical history.', '', '', ''),
			'318' =>  array('318', 'X-rays/radiology films', '', '', ''),
			'319' =>  array('319', 'Pre/post-operative x-rays/photographs.', '', '', ''),
			'320' =>  array('320', 'Study models.', '', '', ''),
			'322' =>  array('322', 'Recent Full Mouth X-rays', '', '', ''),
			'323' =>  array('323', 'Study models, x-rays, and/or narrative.', '', '', ''),
			'324' =>  array('324', 'Recent x-ray of treatment area and/or narrative.', '', '', ''),
			'325' =>  array('325', 'Recent fm x-rays and/or narrative.', '', '', ''),
			'326' =>  array('326', 'Copy of transplant acquisition invoice.', '', '', ''),
			'327' =>  array('327', 'Periodontal case type diagnosis and recent pocket depth chart with narrative.', '', '', ''),
			'329' =>  array('329', 'Exercise notes.', '', '', ''),
			'330' =>  array('330', 'Occupational notes.', '', '', ''),
			'331' =>  array('331', 'History and physical.', '', '', ''),
			'333' =>  array('333', 'Patient release of information authorization.', '', '', ''),
			'334' =>  array('334', 'Oxygen certification.', '', '', ''),
			'335' =>  array('335', 'Durable medical equipment certification.', '', '', ''),
			'336' =>  array('336', 'Chiropractic certification.', '', '', ''),
			'337' =>  array('337', 'Ambulance certification/documentation.', '', '', ''),
			'339' =>  array('339', 'Enteral/parenteral certification.', '', '', ''),
			'340' =>  array('340', 'Pacemaker certification.', '', '', ''),
			'341' =>  array('341', 'Private duty nursing certification.', '', '', ''),
			'342' =>  array('342', 'Podiatric certification.', '', '', ''),
			'343' =>  array('343', 'Documentation that facility is state licensed and Medicare approved as a surgical facility.', '', '', ''),
			'344' =>  array('344', 'Documentation that provider of physical therapy is Medicare Part B approved.', '', '', ''),
			'345' =>  array('345', 'Treatment plan for service/diagnosis', '', '', ''),
			'346' =>  array('346', 'Proposed treatment plan for next 6 months.', '', '', ''),
			'352' =>  array('352', 'Duration of treatment plan.', '', '', ''),
			'353' =>  array('353', 'Orthodontics treatment plan.', '', '', ''),
			'354' =>  array('354', 'Treatment plan for replacement of remaining missing teeth.', '', '', ''),
			'360' =>  array('360', 'Benefits Assignment Certification Indicator', '', '', ''),
			'363' =>  array('363', 'Possible Workers Compensation', '', '', ''),
			'364' =>  array('364', 'Is accident/illness/condition employment related?', '', '', ''),
			'365' =>  array('365', 'Is service the result of an accident?', '', '', ''),
			'366' =>  array('366', 'Is injury due to auto accident?', '', '', ''),
			'374' =>  array('374', 'Is prescribed lenses a result of cataract surgery?', '', '', ''),
			'375' =>  array('375', 'Was refraction performed?', '', '', ''),
			'380' =>  array('380', 'CRNA supervision/medical direction.', '', '', ''),
			'382' =>  array('382', 'Did provider authorize generic or brand name dispensing?', '', '', ''),
			'383' =>  array('383', 'Nerve block use ', '', '', ''),
			'384' =>  array('384', 'Is prosthesis/crown/inlay placement an initial placement or a replacement?', '', '', ''),
			'385' =>  array('385', 'Is appliance upper or lower arch & is appliance fixed or removable?', '', '', ''),
			'386' =>  array('386', 'Orthodontic Treatment/Purpose Indicator', '', '', ''),
			'387' =>  array('387', 'Date patient last examined by entity.', '', '', ''),
			'388' =>  array('388', 'Date post-operative care assumed', '', '', ''),
			'389' =>  array('389', 'Date post-operative care relinquished', '', '', ''),
			'390' =>  array('390', 'Date of most recent medical event necessitating service(s)', '', '', ''),
			'391' =>  array('391', 'Date(s) dialysis conducted', '', '', ''),
			'394' =>  array('394', 'Date(s) of most recent hospitalization related to service', '', '', ''),
			'395' =>  array('395', 'Date entity signed certification/recertification Note: This code requires use of an Entity Code.', '', '', ''),
			'396' =>  array('396', 'Date home dialysis began', '', '', ''),
			'397' =>  array('397', 'Date of onset/exacerbation of illness/condition', '', '', ''),
			'398' =>  array('398', 'Visual field test results', '', '', ''),
			'400' =>  array('400', 'Claim is out of balance', '', '', ''),
			'401' =>  array('401', 'Source of payment is not valid', '', '', ''),
			'402' =>  array('402', 'Amount must be greater than zero.', '', '', ''),
			'403' =>  array('403', 'Entity referral notes/orders/prescription', '', '', ''),
			'406' =>  array('406', 'Brief medical history as related to service(s)', '', '', ''),
			'407' =>  array('407', 'Complications/mitigating circumstances', '', '', ''),
			'408' =>  array('408', 'Initial certification', '', '', ''),
			'409' =>  array('409', 'Medication logs/records ', '', '', ''),
			'414' =>  array('414', 'Necessity for concurrent care ', '', '', ''),
			'417' =>  array('417', 'Prior testing, including result(s) and date(s) as related to service(s)', '', '', ''),
			'419' =>  array('419', 'Individual test(s) comprising the panel and the charges for each test', '', '', ''),
			'420' =>  array('420', 'Name, dosage and medical justification of contrast material used for radiology procedure', '', '', ''),
			'428' =>  array('428', 'Reason for transport by ambulance', '', '', ''),
			'430' =>  array('430', 'Nearest appropriate facility', '', '', ''),
			'431' =>  array('431', 'Patient\'s condition/functional status at time of service.', '', '', ''),
			'432' =>  array('432', 'Date benefits exhausted', '', '', ''),
			'433' =>  array('433', 'Copy of patient revocation of hospice benefits', '', '', ''),
			'434' =>  array('434', 'Reasons for more than one transfer per entitlement period', '', '', ''),
			'435' =>  array('435', 'Notice of Admission', '', '', ''),
			'441' =>  array('441', 'Entity professional qualification for service(s)', '', '', ''),
			'442' =>  array('442', 'Modalities of service', '', '', ''),
			'443' =>  array('443', 'Initial evaluation report', '', '', ''),
			'449' =>  array('449', 'Projected date to discontinue service(s)', '', '', ''),
			'450' =>  array('450', 'Awaiting spend down determination', '', '', ''),
			'451' =>  array('451', 'Preoperative and post-operative diagnosis', '', '', ''),
			'452' =>  array('452', 'Total visits in total number of hours/day and total number of hours/week', '', '', ''),
			'453' =>  array('453', 'Procedure Code Modifier(s) for Service(s) Rendered', '', '', ''),
			'454' =>  array('454', 'Procedure code for services rendered.', '', '', ''),
			'455' =>  array('455', 'Revenue code for services rendered.', '', '', ''),
			'456' =>  array('456', 'Covered Day(s)', '', '', ''),
			'457' =>  array('457', 'Non-Covered Day(s)', '', '', ''),
			'458' =>  array('458', 'Coinsurance Day(s)', '', '', ''),
			'459' =>  array('459', 'Lifetime Reserve Day(s)', '', '', ''),
			'460' =>  array('460', 'NUBC Condition Code(s)', '', '', ''),
			'464' =>  array('464', 'Payer Assigned Claim Control Number', '', '', ''),
			'465' =>  array('465', 'Principal Procedure Code for Service(s) Rendered', '', '', ''),
			'466' =>  array('466', 'Entity\'s Original Signature.', '', '', ''),
			'467' =>  array('467', 'Entity Signature Date.', '', '', ''),
			'468' =>  array('468', 'Patient Signature Source', '', '', ''),
			'469' =>  array('469', 'Purchase Service Charge', '', '', ''),
			'470' =>  array('470', 'Was service purchased from another entity? Note: This code requires use of an Entity Code.', '', '', ''),
			'471' =>  array('471', 'Were services related to an emergency?', '', '', ''),
			'472' =>  array('472', 'Ambulance Run Sheet', '', '', ''),
			'473' =>  array('473', 'Missing or invalid lab indicator', '', '', ''),
			'474' =>  array('474', 'Procedure code and patient gender mismatch', '', '', ''),
			'475' =>  array('475', 'Procedure code not valid for patient age', '', '', ''),
			'476' =>  array('476', 'Missing or invalid units of service', '', '', ''),
			'477' =>  array('477', 'Diagnosis code pointer is missing or invalid', '', '', ''),
			'478' =>  array('478', 'Claim submitter\'s identifier', '', '', ''),
			'479' =>  array('479', 'Other Carrier payer ID is missing or invalid', '', '', ''),
			'480' =>  array('480', 'Entity\'s claim filing indicator.', '', '', ''),
			'481' =>  array('481', 'Claim/submission format is invalid.', '', '', ''),
			'483' =>  array('483', 'Maximum coverage amount met or exceeded for benefit period.', '', '', ''),
			'484' =>  array('484', 'Business Application Currently Not Available', '', '', ''),
			'485' =>  array('485', 'More information available than can be returned in real time mode.', '', '', ''),
			'486' =>  array('486', 'Principal Procedure Date', '', '', ''),
			'487' =>  array('487', 'Claim not found, claim should have been submitted to/through entityName.', '', '', ''),
			'488' =>  array('488', 'Diagnosis code(s) for the services rendered.', '', '', ''),
			'489' =>  array('489', 'Attachment Control Number', '', '', ''),
			'490' =>  array('490', 'Other Procedure Code for Service(s) Rendered', '', '', ''),
			'491' =>  array('491', 'Entity not eligible for encounter submission.', '', '', ''),
			'492' =>  array('492', 'Other Procedure Date', '', '', ''),
			'493' =>  array('493', 'Version/Release/Industry ID code not currently supported by information holder', '', '', ''),
			'494' =>  array('494', 'Real-Time requests not supported by the information holder, resubmit as batch request', '', '', ''),
			'495' =>  array('495', 'Requests for re-adjudication must reference the newly assigned payer claim control number for this previously adjusted claim.', '', '', ''),
			'496' =>  array('496', 'Submitter not approved for electronic claim submissions on behalf of this entity.', '', '', ''),
			'497' =>  array('497', 'Sales tax not paid', '', '', ''),
			'498' =>  array('498', 'Maximum leave days exhausted', '', '', ''),
			'499' =>  array('499', 'No rate on file with the payer for this service for this entity Note: This code requires use of an Entity Code.', '', '', ''),
			'500' =>  array('500', 'Entity\'s Postal/Zip Code.', '', '', ''),
			'501' =>  array('501', 'Entity\'s State/Province.', '', '', ''),
			'502' =>  array('502', 'Entity\'s City.', '', '', ''),
			'503' =>  array('503', 'Entity\'s Street Address.', '', '', ''),
			'504' =>  array('504', 'Entity\'s Last Name.', '', '', ''),
			'505' =>  array('505', 'Entity\'s First Name.', '', '', ''),
			'506' =>  array('506', 'Entity is changing processor/clearinghouse.', '', '', ''),
			'507' =>  array('507', 'HCPCS', '', '', ''),
			'508' =>  array('508', 'ICD9 NOTE: At least one other status code is required to identify the related procedure code or diagnosis code.', '', '', ''),
			'509' =>  array('509', 'External Cause of Injury Code ', '', '', ''),
			'510' =>  array('510', 'Future date.', '', '', ''),
			'511' =>  array('511', 'Invalid character.', '', '', ''),
			'512' =>  array('512', 'Length invalid for receiver\'s application system.', '', '', ''),
			'513' =>  array('513', 'HIPPS Rate Code for services Rendered', '', '', ''),
			'514' =>  array('514', 'Entity\'s Middle Name Note: This code requires use of an Entity Code.', '', '', ''),
			'515' =>  array('515', 'Managed Care review', '', '', ''),
			'516' =>  array('516', 'Other Entity\'s Adjudication or Payment/Remittance Date.', '', '', ''),
			'517' =>  array('517', 'Adjusted Repriced Claim Reference Number', '', '', ''),
			'518' =>  array('518', 'Adjusted Repriced Line item Reference Number', '', '', ''),
			'519' =>  array('519', 'Adjustment Amount', '', '', ''),
			'520' =>  array('520', 'Adjustment Quantity', '', '', ''),
			'521' =>  array('521', 'Adjustment Reason Code', '', '', ''),
			'522' =>  array('522', 'Anesthesia Modifying Units', '', '', ''),
			'523' =>  array('523', 'Anesthesia Unit Count', '', '', ''),
			'524' =>  array('524', 'Arterial Blood Gas Quantity', '', '', ''),
			'525' =>  array('525', 'Begin Therapy Date', '', '', ''),
			'526' =>  array('526', 'Bundled or Unbundled Line Number', '', '', ''),
			'527' =>  array('527', 'Certification Condition Indicator', '', '', ''),
			'528' =>  array('528', 'Certification Period Projected Visit Count', '', '', ''),
			'529' =>  array('529', 'Certification Revision Date', '', '', ''),
			'530' =>  array('530', 'Claim Adjustment Indicator', '', '', ''),
			'531' =>  array('531', 'Claim Disproportinate Share Amount', '', '', ''),
			'532' =>  array('532', 'Claim DRG Amount', '', '', ''),
			'533' =>  array('533', 'Claim DRG Outlier Amount', '', '', ''),
			'534' =>  array('534', 'Claim ESRD Payment Amount', '', '', ''),
			'535' =>  array('535', 'Claim Frequency Code', '', '', ''),
			'536' =>  array('536', 'Claim Indirect Teaching Amount', '', '', ''),
			'537' =>  array('537', 'Claim MSP Pass-through Amount', '', '', ''),
			'538' =>  array('538', 'Claim or Encounter Identifier', '', '', ''),
			'539' =>  array('539', 'Claim PPS Capital Amount', '', '', ''),
			'540' =>  array('540', 'Claim PPS Capital Outlier Amount', '', '', ''),
			'541' =>  array('541', 'Claim Submission Reason Code', '', '', ''),
			'542' =>  array('542', 'Claim Total Denied Charge Amount', '', '', ''),
			'543' =>  array('543', 'Clearinghouse or Value Added Network Trace', '', '', ''),
			'544' =>  array('544', 'Clinical Laboratory Improvement Amendment', '', '', ''),
			'545' =>  array('545', 'Contract Amount', '', '', ''),
			'546' =>  array('546', 'Contract Code', '', '', ''),
			'547' =>  array('547', 'Contract Percentage', '', '', ''),
			'548' =>  array('548', 'Contract Type Code', '', '', ''),
			'549' =>  array('549', 'Contract Version Identifier', '', '', ''),
			'550' =>  array('550', 'Coordination of Benefits Code', '', '', ''),
			'551' =>  array('551', 'Coordination of Benefits Total Submitted Charge', '', '', ''),
			'552' =>  array('552', 'Cost Report Day Count', '', '', ''),
			'553' =>  array('553', 'Covered Amount', '', '', ''),
			'554' =>  array('554', 'Date Claim Paid', '', '', ''),
			'555' =>  array('555', 'Delay Reason Code', '', '', ''),
			'556' =>  array('556', 'Demonstration Project Identifier', '', '', ''),
			'557' =>  array('557', 'Diagnosis Date', '', '', ''),
			'558' =>  array('558', 'Discount Amount', '', '', ''),
			'559' =>  array('559', 'Document Control Identifier', '', '', ''),
			'560' =>  array('560', 'Entity\'s Additional/Secondary Identifier.', '', '', ''),
			'561' =>  array('561', 'Entity\'s Contact Name.', '', '', ''),
			'562' =>  array('562', 'Entity\'s National Provider Identifier ', '', '', ''),
			'563' =>  array('563', 'Entity\'s Tax Amount.', '', '', ''),
			'564' =>  array('564', 'EPSDT Indicator', '', '', ''),
			'565' =>  array('565', 'Estimated Claim Due Amount', '', '', ''),
			'566' =>  array('566', 'Exception Code', '', '', ''),
			'567' =>  array('567', 'Facility Code Qualifier', '', '', ''),
			'568' =>  array('568', 'Family Planning Indicator', '', '', ''),
			'569' =>  array('569', 'Fixed Format Information', '', '', ''),
			'570' =>  array('570', 'Free Form Message Text', '', '', ''),
			'571' =>  array('571', 'Frequency Count', '', '', ''),
			'572' =>  array('572', 'Frequency Period', '', '', ''),
			'573' =>  array('573', 'Functional Limitation Code', '', '', ''),
			'574' =>  array('574', 'HCPCS Payable Amount Home Health', '', '', ''),
			'575' =>  array('575', 'Homebound Indicator', '', '', ''),
			'576' =>  array('576', 'Immunization Batch Number', '', '', ''),
			'577' =>  array('577', 'Industry Code', '', '', ''),
			'578' =>  array('578', 'Insurance Type Code', '', '', ''),
			'579' =>  array('579', 'Investigational Device Exemption Identifier', '', '', ''),
			'580' =>  array('580', 'Last Certification Date', '', '', ''),
			'581' =>  array('581', 'Last Worked Date', '', '', ''),
			'582' =>  array('582', 'Lifetime Psychiatric Days Count', '', '', ''),
			'583' =>  array('583', 'Line Item Charge Amount', '', '', ''),
			'584' =>  array('584', 'Line Item Control Number', '', '', ''),
			'585' =>  array('585', 'Denied Charge or Non-covered Charge', '', '', ''),
			'586' =>  array('586', 'Line Note Text', '', '', ''),
			'587' =>  array('587', 'Measurement Reference Identification Code', '', '', ''),
			'588' =>  array('588', 'Medical Record Number', '', '', ''),
			'589' =>  array('589', 'Provider Accept Assignment Code', '', '', ''),
			'590' =>  array('590', 'Medicare Coverage Indicator', '', '', ''),
			'591' =>  array('591', 'Medicare Paid at 100% Amount', '', '', ''),
			'592' =>  array('592', 'Medicare Paid at 80% Amount', '', '', ''),
			'593' =>  array('593', 'Medicare Section 4081 Indicator', '', '', ''),
			'594' =>  array('594', 'Mental Status Code', '', '', ''),
			'595' =>  array('595', 'Monthly Treatment Count', '', '', ''),
			'596' =>  array('596', 'Non-covered Charge Amount', '', '', ''),
			'597' =>  array('597', 'Non-payable Professional Component Amount', '', '', ''),
			'598' =>  array('598', 'Non-payable Professional Component Billed Amount', '', '', ''),
			'599' =>  array('599', 'Note Reference Code', '', '', ''),
			'600' =>  array('600', 'Oxygen Saturation Qty', '', '', ''),
			'601' =>  array('601', 'Oxygen Test Condition Code', '', '', ''),
			'602' =>  array('602', 'Oxygen Test Date', '', '', ''),
			'603' =>  array('603', 'Old Capital Amount', '', '', ''),
			'604' =>  array('604', 'Originator Application Transaction Identifier', '', '', ''),
			'605' =>  array('605', 'Orthodontic Treatment Months Count', '', '', ''),
			'606' =>  array('606', 'Paid From Part A Medicare Trust Fund Amount', '', '', ''),
			'607' =>  array('607', 'Paid From Part B Medicare Trust Fund Amount', '', '', ''),
			'608' =>  array('608', 'Paid Service Unit Count', '', '', ''),
			'609' =>  array('609', 'Participation Agreement', '', '', ''),
			'610' =>  array('610', 'Patient Discharge Facility Type Code', '', '', ''),
			'611' =>  array('611', 'Peer Review Authorization Number', '', '', ''),
			'612' =>  array('612', 'Per Day Limit Amount', '', '', ''),
			'613' =>  array('613', 'Physician Contact Date', '', '', ''),
			'614' =>  array('614', 'Physician Order Date', '', '', ''),
			'615' =>  array('615', 'Policy Compliance Code', '', '', ''),
			'616' =>  array('616', 'Policy Name', '', '', ''),
			'617' =>  array('617', 'Postage Claimed Amount', '', '', ''),
			'618' =>  array('618', 'PPS-Capital DSH DRG Amount', '', '', ''),
			'619' =>  array('619', 'PPS-Capital Exception Amount', '', '', ''),
			'620' =>  array('620', 'PPS-Capital FSP DRG Amount', '', '', ''),
			'621' =>  array('621', 'PPS-Capital HSP DRG Amount', '', '', ''),
			'622' =>  array('622', 'PPS-Capital IME Amount', '', '', ''),
			'623' =>  array('623', 'PPS-Operating Federal Specific DRG Amount', '', '', ''),
			'624' =>  array('624', 'PPS-Operating Hospital Specific DRG Amount', '', '', ''),
			'625' =>  array('625', 'Predetermination of Benefits Identifier', '', '', ''),
			'626' =>  array('626', 'Pregnancy Indicator', '', '', ''),
			'627' =>  array('627', 'Pre-Tax Claim Amount', '', '', ''),
			'628' =>  array('628', 'Pricing Methodology', '', '', ''),
			'629' =>  array('629', 'Property Casualty Claim Number', '', '', ''),
			'630' =>  array('630', 'Referring CLIA Number', '', '', ''),
			'631' =>  array('631', 'Reimbursement Rate', '', '', ''),
			'632' =>  array('632', 'Reject Reason Code', '', '', ''),
			'633' =>  array('633', 'Related Causes Code ', '', '', ''),
			'634' =>  array('634', 'Remark Code', '', '', ''),
			'635' =>  array('635', 'Repriced Ambulatory Patient Group Code', '', '', ''),
			'636' =>  array('636', 'Repriced Line Item Reference Number', '', '', ''),
			'637' =>  array('637', 'Repriced Saving Amount', '', '', ''),
			'638' =>  array('638', 'Repricing Per Diem or Flat Rate Amount', '', '', ''),
			'639' =>  array('639', 'Responsibility Amount', '', '', ''),
			'640' =>  array('640', 'Sales Tax Amount', '', '', ''),
			'642' =>  array('642', 'Service Authorization Exception Code', '', '', ''),
			'643' =>  array('643', 'Service Line Paid Amount', '', '', ''),
			'644' =>  array('644', 'Service Line Rate', '', '', ''),
			'645' =>  array('645', 'Service Tax Amount', '', '', ''),
			'646' =>  array('646', 'Ship, Delivery or Calendar Pattern Code', '', '', ''),
			'647' =>  array('647', 'Shipped Date', '', '', ''),
			'648' =>  array('648', 'Similar Illness or Symptom Date', '', '', ''),
			'649' =>  array('649', 'Skilled Nursing Facility Indicator', '', '', ''),
			'650' =>  array('650', 'Special Program Indicator', '', '', ''),
			'651' =>  array('651', 'State Industrial Accident Provider Number', '', '', ''),
			'652' =>  array('652', 'Terms Discount Percentage', '', '', ''),
			'653' =>  array('653', 'Test Performed Date', '', '', ''),
			'654' =>  array('654', 'Total Denied Charge Amount', '', '', ''),
			'655' =>  array('655', 'Total Medicare Paid Amount', '', '', ''),
			'656' =>  array('656', 'Total Visits Projected This Certification Count', '', '', ''),
			'657' =>  array('657', 'Total Visits Rendered Count', '', '', ''),
			'658' =>  array('658', 'Treatment Code', '', '', ''),
			'659' =>  array('659', 'Unit or Basis for Measurement Code', '', '', ''),
			'660' =>  array('660', 'Universal Product Number', '', '', ''),
			'661' =>  array('661', 'Visits Prior to Recertification Date Count CR702', '', '', ''),
			'662' =>  array('662', 'X-ray Availability Indicator', '', '', ''),
			'663' =>  array('663', 'Entity\'s Group Name.', '', '', ''),
			'664' =>  array('664', 'Orthodontic Banding Date', '', '', ''),
			'665' =>  array('665', 'Surgery Date', '', '', ''),
			'666' =>  array('666', 'Surgical Procedure Code', '', '', ''),
			'667' =>  array('667', 'Real-Time requests not supported by the information holder, do not resubmit', '', '', ''),
			'668' =>  array('668', 'Missing Endodontics treatment history and prognosis', '', '', ''),
			'669' =>  array('669', 'Dental service narrative needed.', '', '', ''),
			'670' =>  array('670', 'Funds applied from a consumer spending account such as consumer directed/driven health plan ', '', '', ''),
			'671' =>  array('671', 'Funds may be available from a consumer spending account such as consumer directed/driven health plan ', '', '', ''),
			'672' =>  array('672', 'Other Payer\'s payment information is out of balance', '', '', ''),
			'673' =>  array('673', 'Patient Reason for Visit', '', '', ''),
			'674' =>  array('674', 'Authorization exceeded', '', '', ''),
			'675' =>  array('675', 'Facility admission through discharge dates', '', '', ''),
			'676' =>  array('676', 'Entity possibly compensated by facility.', '', '', ''),
			'677' =>  array('677', 'Entity not affiliated.', '', '', ''),
			'678' =>  array('678', 'Revenue code and patient gender mismatch', '', '', ''),
			'679' =>  array('679', 'Submit newborn services on mother\'s claim', '', '', ''),
			'680' =>  array('680', 'Entity\'s Country.', '', '', ''),
			'681' =>  array('681', 'Claim currency not supported', '', '', ''),
			'682' =>  array('682', 'Cosmetic procedure', '', '', ''),
			'683' =>  array('683', 'Awaiting Associated Hospital Claims', '', '', ''),
			'684' =>  array('684', 'Rejected.', '', '', ''),
			'685' =>  array('685', 'Claim could not complete adjudication in real time.', '', '', ''),
			'686' =>  array('686', 'The claim/ encounter has completed the adjudication cycle and the entire claim has been voided', '', '', ''),
			'687' =>  array('687', 'Claim estimation can not be completed in real time.', '', '', ''),
			'688' =>  array('688', 'Present on Admission Indicator for reported diagnosis code(s).', '', '', ''),
			'689' =>  array('689', 'Entity was unable to respond within the expected time frame.', '', '', ''),
			'690' =>  array('690', 'Multiple claims or estimate requests cannot be processed in real time.', '', '', ''),
			'691' =>  array('691', 'Multiple claim status requests cannot be processed in real time.', '', '', ''),
			'692' =>  array('692', 'Contracted funding agreement-Subscriber is employed by the provider of services', '', '', ''),
			'693' =>  array('693', 'Amount must be greater than or equal to zero.', '', '', ''),
			'694' =>  array('694', 'Amount must not be equal to zero.', '', '', ''),
			'695' =>  array('695', 'Entity\'s Country Subdivision Code.', '', '', ''),
			'696' =>  array('696', 'Claim Adjustment Group Code.', '', '', ''),
			'697' =>  array('697', 'Invalid Decimal Precision.', '', '', ''),
			'698' =>  array('698', 'Form Type Identification', '', '', ''),
			'699' =>  array('699', 'Question/Response from Supporting Documentation Form', '', '', ''),
			'700' =>  array('700', 'ICD10.', '', '', ''),
			'701' =>  array('701', 'Initial Treatment Date', '', '', ''),
			'702' =>  array('702', 'Repriced Claim Reference Number', '', '', ''),
			'703' =>  array('703', 'Advanced Billing Concepts ', '', '', ''),
			'704' =>  array('704', 'Claim Note Text', '', '', ''),
			'705' =>  array('705', 'Repriced Allowed Amount', '', '', ''),
			'706' =>  array('706', 'Repriced Approved Amount', '', '', ''),
			'707' =>  array('707', 'Repriced Approved Ambulatory Patient Group Amount', '', '', ''),
			'708' =>  array('708', 'Repriced Approved Revenue Code', '', '', ''),
			'709' =>  array('709', 'Repriced Approved Service Unit Count', '', '', ''),
			'710' =>  array('710', 'Line Adjudication Information.', '', '', ''),
			'711' =>  array('711', 'Stretcher purpose', '', '', ''),
			'712' =>  array('712', 'Obstetric Additional Units', '', '', ''),
			'713' =>  array('713', 'Patient Condition Description', '', '', ''),
			'714' =>  array('714', 'Care Plan Oversight Number', '', '', ''),
			'715' =>  array('715', 'Acute Manifestation Date', '', '', ''),
			'716' =>  array('716', 'Repriced Approved DRG Code', '', '', ''),
			'717' =>  array('717', 'This claim has been split for processing.', '', '', ''),
			'718' =>  array('718', 'Claim/service not submitted within the required timeframe ', '', '', ''),
			'719' =>  array('719', 'NUBC Occurrence Code(s)', '', '', ''),
			'720' =>  array('720', 'NUBC Occurrence Code Date(s)', '', '', ''),
			'721' =>  array('721', 'NUBC Occurrence Span Code(s)', '', '', ''),
			'722' =>  array('722', 'NUBC Occurrence Span Code Date(s)', '', '', ''),
			'723' =>  array('723', 'Drug days supply', '', '', ''),
			'724' =>  array('724', 'Drug dosage', '', '', ''),
			'725' =>  array('725', 'NUBC Value Code(s)', '', '', ''),
			'726' =>  array('726', 'NUBC Value Code Amount(s)', '', '', ''),
			'727' =>  array('727', 'Accident date', '', '', ''),
			'728' =>  array('728', 'Accident state', '', '', ''),
			'729' =>  array('729', 'Accident description', '', '', ''),
			'730' =>  array('730', 'Accident cause', '', '', ''),
			'731' =>  array('731', 'Measurement value/test result', '', '', ''),
			'732' =>  array('732', 'Information submitted inconsistent with billing guidelines.', '', '', ''),
			'733' =>  array('733', 'Prefix for entity\'s contract/member number.', '', '', ''),
			'734' =>  array('734', 'Verifying premium payment', '', '', ''),
			'735' =>  array('735', 'This service/claim is included in the allowance for another service or claim.', '', '', ''),
			'736' =>  array('736', 'A related or qualifying service/claim has not been received/adjudicated.', '', '', ''),
			'737' =>  array('737', 'Current Dental Terminology ', '', '', ''),
			'738' =>  array('738', 'Home Infusion EDI Coalition ', '', '', ''),
			'739' =>  array('739', 'Jurisdiction Specific Procedure or Supply Code', '', '', ''),
			'740' =>  array('740', 'Drop-Off Location', '', '', ''),
			'741' =>  array('741', 'Entity must be a person.', '', '', ''),
			'742' =>  array('742', 'Payer Responsibility Sequence Number Code', '', '', ''),
			'743' =>  array('743', 'Entity’s credential/enrollment information.', '', '', ''),
			'744' =>  array('744', 'Services/charges related to the treatment of a hospital-acquired condition or preventable medical error.', '', '', ''),
			'745' =>  array('745', 'Identifier Qualifier Note: At least one other status code is required to identify the specific identifier qualifier in error.', '', '', ''),
			'746' =>  array('746', 'Duplicate Submission Note: use only at the information receiver level in the Health Care Claim Acknowledgement transaction.', '', '', ''),
			'747' =>  array('747', 'Hospice Employee Indicator', '', '', ''),
			'748' =>  array('748', 'Corrected Data Note: Requires a second status code to identify the corrected data.', '', '', ''),
			'749' =>  array('749', 'Date of Injury/Illness', '', '', ''),
			'750' =>  array('750', 'Auto Accident State or Province Code', '', '', ''),
			'751' =>  array('751', 'Ambulance Pick-up State or Province Code', '', '', ''),
			'752' =>  array('752', 'Ambulance Drop-off State or Province Code', '', '', ''),
			'753' =>  array('753', 'Co-pay status code.', '', '', ''),
			'754' =>  array('754', 'Entity Name Suffix.', '', '', ''),
			'755' =>  array('755', 'Entity\'s primary identifier.', '', '', ''),
			'756' =>  array('756', 'Entity\'s Received Date.', '', '', ''),
			'757' =>  array('757', 'Last seen date.', '', '', ''),
			'758' =>  array('758', 'Repriced approved HCPCS code.', '', '', ''),
			'759' =>  array('759', 'Round trip purpose description.', '', '', ''),
			'760' =>  array('760', 'Tooth status code.', '', '', ''),
			'761' =>  array('761', 'Entity\'s referral number.', '', '', '')
			);
			if (array_key_exists($code, $this->STC_Status_Code)){
				return $this->STC_Status_Code[$code];
			} else {
				return array($code, "Not found in STC Status codes table", "", "", "");
			}
		}
	} // end get_STC_Status_Code
	
} // end class


?>
